@@ -1,10 +1,11 @@
 from esphome import pins
 import esphome.codegen as cg
-from esphome.components.esp32 import get_esp32_variant
+from esphome.components.esp32 import VARIANT_ESP32P4, get_esp32_variant
 from esphome.components.esp32.const import (
     VARIANT_ESP32,
     VARIANT_ESP32C2,
     VARIANT_ESP32C3,
+    VARIANT_ESP32C5,
     VARIANT_ESP32C6,
     VARIANT_ESP32H2,
     VARIANT_ESP32S2,
@@ -51,82 +52,103 @@ SAMPLING_MODES = {
     "max": sampling_mode.MAX,
 }
 
-adc1_channel_t = cg.global_ns.enum("adc1_channel_t")
-adc2_channel_t = cg.global_ns.enum("adc2_channel_t")
+adc_unit_t = cg.global_ns.enum("adc_unit_t", is_class=True)
+
+adc_channel_t = cg.global_ns.enum("adc_channel_t", is_class=True)
 
 # pin to adc1 channel mapping
 # https://github.com/espressif/esp-idf/blob/v4.4.8/components/driver/include/driver/adc.h
 ESP32_VARIANT_ADC1_PIN_TO_CHANNEL = {
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32/include/soc/adc_channel.h
     VARIANT_ESP32: {
-        36: adc1_channel_t.ADC1_CHANNEL_0,
-        37: adc1_channel_t.ADC1_CHANNEL_1,
-        38: adc1_channel_t.ADC1_CHANNEL_2,
-        39: adc1_channel_t.ADC1_CHANNEL_3,
-        32: adc1_channel_t.ADC1_CHANNEL_4,
-        33: adc1_channel_t.ADC1_CHANNEL_5,
-        34: adc1_channel_t.ADC1_CHANNEL_6,
-        35: adc1_channel_t.ADC1_CHANNEL_7,
+        36: adc_channel_t.ADC_CHANNEL_0,
+        37: adc_channel_t.ADC_CHANNEL_1,
+        38: adc_channel_t.ADC_CHANNEL_2,
+        39: adc_channel_t.ADC_CHANNEL_3,
+        32: adc_channel_t.ADC_CHANNEL_4,
+        33: adc_channel_t.ADC_CHANNEL_5,
+        34: adc_channel_t.ADC_CHANNEL_6,
+        35: adc_channel_t.ADC_CHANNEL_7,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32c2/include/soc/adc_channel.h
     VARIANT_ESP32C2: {
-        0: adc1_channel_t.ADC1_CHANNEL_0,
-        1: adc1_channel_t.ADC1_CHANNEL_1,
-        2: adc1_channel_t.ADC1_CHANNEL_2,
-        3: adc1_channel_t.ADC1_CHANNEL_3,
-        4: adc1_channel_t.ADC1_CHANNEL_4,
+        0: adc_channel_t.ADC_CHANNEL_0,
+        1: adc_channel_t.ADC_CHANNEL_1,
+        2: adc_channel_t.ADC_CHANNEL_2,
+        3: adc_channel_t.ADC_CHANNEL_3,
+        4: adc_channel_t.ADC_CHANNEL_4,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32c3/include/soc/adc_channel.h
     VARIANT_ESP32C3: {
-        0: adc1_channel_t.ADC1_CHANNEL_0,
-        1: adc1_channel_t.ADC1_CHANNEL_1,
-        2: adc1_channel_t.ADC1_CHANNEL_2,
-        3: adc1_channel_t.ADC1_CHANNEL_3,
-        4: adc1_channel_t.ADC1_CHANNEL_4,
+        0: adc_channel_t.ADC_CHANNEL_0,
+        1: adc_channel_t.ADC_CHANNEL_1,
+        2: adc_channel_t.ADC_CHANNEL_2,
+        3: adc_channel_t.ADC_CHANNEL_3,
+        4: adc_channel_t.ADC_CHANNEL_4,
+    },
+    # ESP32-C5 ADC1 pin mapping - based on official ESP-IDF documentation
+    # https://docs.espressif.com/projects/esp-idf/en/latest/esp32c5/api-reference/peripherals/gpio.html
+    VARIANT_ESP32C5: {
+        1: adc_channel_t.ADC_CHANNEL_0,
+        2: adc_channel_t.ADC_CHANNEL_1,
+        3: adc_channel_t.ADC_CHANNEL_2,
+        4: adc_channel_t.ADC_CHANNEL_3,
+        5: adc_channel_t.ADC_CHANNEL_4,
+        6: adc_channel_t.ADC_CHANNEL_5,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32c6/include/soc/adc_channel.h
     VARIANT_ESP32C6: {
-        0: adc1_channel_t.ADC1_CHANNEL_0,
-        1: adc1_channel_t.ADC1_CHANNEL_1,
-        2: adc1_channel_t.ADC1_CHANNEL_2,
-        3: adc1_channel_t.ADC1_CHANNEL_3,
-        4: adc1_channel_t.ADC1_CHANNEL_4,
-        5: adc1_channel_t.ADC1_CHANNEL_5,
-        6: adc1_channel_t.ADC1_CHANNEL_6,
+        0: adc_channel_t.ADC_CHANNEL_0,
+        1: adc_channel_t.ADC_CHANNEL_1,
+        2: adc_channel_t.ADC_CHANNEL_2,
+        3: adc_channel_t.ADC_CHANNEL_3,
+        4: adc_channel_t.ADC_CHANNEL_4,
+        5: adc_channel_t.ADC_CHANNEL_5,
+        6: adc_channel_t.ADC_CHANNEL_6,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32h2/include/soc/adc_channel.h
     VARIANT_ESP32H2: {
-        1: adc1_channel_t.ADC1_CHANNEL_0,
-        2: adc1_channel_t.ADC1_CHANNEL_1,
-        3: adc1_channel_t.ADC1_CHANNEL_2,
-        4: adc1_channel_t.ADC1_CHANNEL_3,
-        5: adc1_channel_t.ADC1_CHANNEL_4,
+        1: adc_channel_t.ADC_CHANNEL_0,
+        2: adc_channel_t.ADC_CHANNEL_1,
+        3: adc_channel_t.ADC_CHANNEL_2,
+        4: adc_channel_t.ADC_CHANNEL_3,
+        5: adc_channel_t.ADC_CHANNEL_4,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32s2/include/soc/adc_channel.h
     VARIANT_ESP32S2: {
-        1: adc1_channel_t.ADC1_CHANNEL_0,
-        2: adc1_channel_t.ADC1_CHANNEL_1,
-        3: adc1_channel_t.ADC1_CHANNEL_2,
-        4: adc1_channel_t.ADC1_CHANNEL_3,
-        5: adc1_channel_t.ADC1_CHANNEL_4,
-        6: adc1_channel_t.ADC1_CHANNEL_5,
-        7: adc1_channel_t.ADC1_CHANNEL_6,
-        8: adc1_channel_t.ADC1_CHANNEL_7,
-        9: adc1_channel_t.ADC1_CHANNEL_8,
-        10: adc1_channel_t.ADC1_CHANNEL_9,
+        1: adc_channel_t.ADC_CHANNEL_0,
+        2: adc_channel_t.ADC_CHANNEL_1,
+        3: adc_channel_t.ADC_CHANNEL_2,
+        4: adc_channel_t.ADC_CHANNEL_3,
+        5: adc_channel_t.ADC_CHANNEL_4,
+        6: adc_channel_t.ADC_CHANNEL_5,
+        7: adc_channel_t.ADC_CHANNEL_6,
+        8: adc_channel_t.ADC_CHANNEL_7,
+        9: adc_channel_t.ADC_CHANNEL_8,
+        10: adc_channel_t.ADC_CHANNEL_9,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32s3/include/soc/adc_channel.h
     VARIANT_ESP32S3: {
-        1: adc1_channel_t.ADC1_CHANNEL_0,
-        2: adc1_channel_t.ADC1_CHANNEL_1,
-        3: adc1_channel_t.ADC1_CHANNEL_2,
-        4: adc1_channel_t.ADC1_CHANNEL_3,
-        5: adc1_channel_t.ADC1_CHANNEL_4,
-        6: adc1_channel_t.ADC1_CHANNEL_5,
-        7: adc1_channel_t.ADC1_CHANNEL_6,
-        8: adc1_channel_t.ADC1_CHANNEL_7,
-        9: adc1_channel_t.ADC1_CHANNEL_8,
-        10: adc1_channel_t.ADC1_CHANNEL_9,
+        1: adc_channel_t.ADC_CHANNEL_0,
+        2: adc_channel_t.ADC_CHANNEL_1,
+        3: adc_channel_t.ADC_CHANNEL_2,
+        4: adc_channel_t.ADC_CHANNEL_3,
+        5: adc_channel_t.ADC_CHANNEL_4,
+        6: adc_channel_t.ADC_CHANNEL_5,
+        7: adc_channel_t.ADC_CHANNEL_6,
+        8: adc_channel_t.ADC_CHANNEL_7,
+        9: adc_channel_t.ADC_CHANNEL_8,
+        10: adc_channel_t.ADC_CHANNEL_9,
+    },
+    VARIANT_ESP32P4: {
+        16: adc_channel_t.ADC_CHANNEL_0,
+        17: adc_channel_t.ADC_CHANNEL_1,
+        18: adc_channel_t.ADC_CHANNEL_2,
+        19: adc_channel_t.ADC_CHANNEL_3,
+        20: adc_channel_t.ADC_CHANNEL_4,
+        21: adc_channel_t.ADC_CHANNEL_5,
+        22: adc_channel_t.ADC_CHANNEL_6,
+        23: adc_channel_t.ADC_CHANNEL_7,
     },
 }
 
@@ -135,54 +157,64 @@ ESP32_VARIANT_ADC1_PIN_TO_CHANNEL = {
 ESP32_VARIANT_ADC2_PIN_TO_CHANNEL = {
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32/include/soc/adc_channel.h
     VARIANT_ESP32: {
-        4: adc2_channel_t.ADC2_CHANNEL_0,
-        0: adc2_channel_t.ADC2_CHANNEL_1,
-        2: adc2_channel_t.ADC2_CHANNEL_2,
-        15: adc2_channel_t.ADC2_CHANNEL_3,
-        13: adc2_channel_t.ADC2_CHANNEL_4,
-        12: adc2_channel_t.ADC2_CHANNEL_5,
-        14: adc2_channel_t.ADC2_CHANNEL_6,
-        27: adc2_channel_t.ADC2_CHANNEL_7,
-        25: adc2_channel_t.ADC2_CHANNEL_8,
-        26: adc2_channel_t.ADC2_CHANNEL_9,
+        4: adc_channel_t.ADC_CHANNEL_0,
+        0: adc_channel_t.ADC_CHANNEL_1,
+        2: adc_channel_t.ADC_CHANNEL_2,
+        15: adc_channel_t.ADC_CHANNEL_3,
+        13: adc_channel_t.ADC_CHANNEL_4,
+        12: adc_channel_t.ADC_CHANNEL_5,
+        14: adc_channel_t.ADC_CHANNEL_6,
+        27: adc_channel_t.ADC_CHANNEL_7,
+        25: adc_channel_t.ADC_CHANNEL_8,
+        26: adc_channel_t.ADC_CHANNEL_9,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32c2/include/soc/adc_channel.h
     VARIANT_ESP32C2: {
-        5: adc2_channel_t.ADC2_CHANNEL_0,
+        5: adc_channel_t.ADC_CHANNEL_0,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32c3/include/soc/adc_channel.h
     VARIANT_ESP32C3: {
-        5: adc2_channel_t.ADC2_CHANNEL_0,
+        5: adc_channel_t.ADC_CHANNEL_0,
     },
+    # ESP32-C5 has no ADC2 channels
+    VARIANT_ESP32C5: {},  # no ADC2
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32c6/include/soc/adc_channel.h
     VARIANT_ESP32C6: {},  # no ADC2
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32h2/include/soc/adc_channel.h
     VARIANT_ESP32H2: {},  # no ADC2
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32s2/include/soc/adc_channel.h
     VARIANT_ESP32S2: {
-        11: adc2_channel_t.ADC2_CHANNEL_0,
-        12: adc2_channel_t.ADC2_CHANNEL_1,
-        13: adc2_channel_t.ADC2_CHANNEL_2,
-        14: adc2_channel_t.ADC2_CHANNEL_3,
-        15: adc2_channel_t.ADC2_CHANNEL_4,
-        16: adc2_channel_t.ADC2_CHANNEL_5,
-        17: adc2_channel_t.ADC2_CHANNEL_6,
-        18: adc2_channel_t.ADC2_CHANNEL_7,
-        19: adc2_channel_t.ADC2_CHANNEL_8,
-        20: adc2_channel_t.ADC2_CHANNEL_9,
+        11: adc_channel_t.ADC_CHANNEL_0,
+        12: adc_channel_t.ADC_CHANNEL_1,
+        13: adc_channel_t.ADC_CHANNEL_2,
+        14: adc_channel_t.ADC_CHANNEL_3,
+        15: adc_channel_t.ADC_CHANNEL_4,
+        16: adc_channel_t.ADC_CHANNEL_5,
+        17: adc_channel_t.ADC_CHANNEL_6,
+        18: adc_channel_t.ADC_CHANNEL_7,
+        19: adc_channel_t.ADC_CHANNEL_8,
+        20: adc_channel_t.ADC_CHANNEL_9,
     },
     # https://github.com/espressif/esp-idf/blob/master/components/soc/esp32s3/include/soc/adc_channel.h
     VARIANT_ESP32S3: {
-        11: adc2_channel_t.ADC2_CHANNEL_0,
-        12: adc2_channel_t.ADC2_CHANNEL_1,
-        13: adc2_channel_t.ADC2_CHANNEL_2,
-        14: adc2_channel_t.ADC2_CHANNEL_3,
-        15: adc2_channel_t.ADC2_CHANNEL_4,
-        16: adc2_channel_t.ADC2_CHANNEL_5,
-        17: adc2_channel_t.ADC2_CHANNEL_6,
-        18: adc2_channel_t.ADC2_CHANNEL_7,
-        19: adc2_channel_t.ADC2_CHANNEL_8,
-        20: adc2_channel_t.ADC2_CHANNEL_9,
+        11: adc_channel_t.ADC_CHANNEL_0,
+        12: adc_channel_t.ADC_CHANNEL_1,
+        13: adc_channel_t.ADC_CHANNEL_2,
+        14: adc_channel_t.ADC_CHANNEL_3,
+        15: adc_channel_t.ADC_CHANNEL_4,
+        16: adc_channel_t.ADC_CHANNEL_5,
+        17: adc_channel_t.ADC_CHANNEL_6,
+        18: adc_channel_t.ADC_CHANNEL_7,
+        19: adc_channel_t.ADC_CHANNEL_8,
+        20: adc_channel_t.ADC_CHANNEL_9,
+    },
+    VARIANT_ESP32P4: {
+        49: adc_channel_t.ADC_CHANNEL_0,
+        50: adc_channel_t.ADC_CHANNEL_1,
+        51: adc_channel_t.ADC_CHANNEL_2,
+        52: adc_channel_t.ADC_CHANNEL_3,
+        53: adc_channel_t.ADC_CHANNEL_4,
+        54: adc_channel_t.ADC_CHANNEL_5,
     },
 }
 
@@ -235,6 +267,11 @@ def validate_adc_pin(value):
             {CONF_ANALOG: True, CONF_INPUT: True}, internal=True
         )(value)
 
+    if CORE.is_nrf52:
+        return pins.gpio_pin_schema(
+            {CONF_ANALOG: True, CONF_INPUT: True}, internal=True
+        )(value)
+
     raise NotImplementedError
 
 
@@ -251,5 +288,6 @@ FILTER_SOURCE_FILES = filter_source_files_from_platform(
             PlatformFramework.RTL87XX_ARDUINO,
             PlatformFramework.LN882X_ARDUINO,
         },
+        "adc_sensor_zephyr.cpp": {PlatformFramework.NRF52_ZEPHYR},
     }
 )
