@@ -12,6 +12,8 @@ from esphome.const import (
     CONF_ROTATION,
     CONF_TO,
     CONF_TRIGGER_ID,
+    CONF_UPDATE_INTERVAL,
+    SCHEDULER_DONT_RUN,
 )
 from esphome.core import coroutine_with_priority
 
@@ -51,8 +53,7 @@ DISPLAY_ROTATIONS = {
 
 def validate_rotation(value):
     value = cv.string(value)
-    if value.endswith("°"):
-        value = value[:-1]
+    value = value.removesuffix("°")
     return cv.enum(DISPLAY_ROTATIONS, int=True)(value)
 
 
@@ -68,6 +69,18 @@ BASIC_DISPLAY_SCHEMA = cv.Schema(
     }
 ).extend(cv.polling_component_schema("1s"))
 
+
+def _validate_test_card(config):
+    if (
+        config.get(CONF_SHOW_TEST_CARD, False)
+        and config.get(CONF_UPDATE_INTERVAL, False) == SCHEDULER_DONT_RUN
+    ):
+        raise cv.Invalid(
+            f"`{CONF_SHOW_TEST_CARD}: True` cannot be used with `{CONF_UPDATE_INTERVAL}: never` because this combination will not show a test_card."
+        )
+    return config
+
+
 FULL_DISPLAY_SCHEMA = BASIC_DISPLAY_SCHEMA.extend(
     {
         cv.Optional(CONF_ROTATION): validate_rotation,
@@ -95,6 +108,7 @@ FULL_DISPLAY_SCHEMA = BASIC_DISPLAY_SCHEMA.extend(
         cv.Optional(CONF_SHOW_TEST_CARD): cv.boolean,
     }
 )
+FULL_DISPLAY_SCHEMA.add_extra(_validate_test_card)
 
 
 async def setup_display_core_(var, config):
@@ -201,7 +215,6 @@ async def display_is_displaying_page_to_code(config, condition_id, template_arg,
     page = await cg.get_variable(config[CONF_PAGE_ID])
     var = cg.new_Pvariable(condition_id, template_arg, paren)
     cg.add(var.set_page(page))
-
     return var
 
 
