@@ -57,8 +57,8 @@ void Inkplate6::setup() {
  * Allocate buffers. May be called after setup to re-initialise if e.g. greyscale is changed.
  */
 void Inkplate6::initialize_() {
-  ExternalRAMAllocator<uint8_t> allocator(ExternalRAMAllocator<uint8_t>::ALLOW_FAILURE);
-  ExternalRAMAllocator<uint32_t> allocator32(ExternalRAMAllocator<uint32_t>::ALLOW_FAILURE);
+  RAMAllocator<uint8_t> allocator;
+  RAMAllocator<uint32_t> allocator32;
   uint32_t buffer_size = this->get_buffer_length_();
   if (buffer_size == 0)
     return;
