@@ -11,12 +11,20 @@
 namespace esphome {
 namespace lvgl {
 
-class LVGLSelect : public select::Select {
+class LVGLSelect : public select::Select, public Component {
  public:
-  void set_widget(LvSelectable *widget, lv_anim_enable_t anim = LV_ANIM_OFF) {
-    this->widget_ = widget;
-    this->anim_ = anim;
+  LVGLSelect(LvSelectable *widget, lv_anim_enable_t anim, bool restore)
+      : widget_(widget), anim_(anim), restore_(restore) {}
+
+  void setup() override {
     this->set_options_();
+    if (this->restore_) {
+      size_t index;
+      this->pref_ = global_preferences->make_preference<size_t>(this->get_object_id_hash());
+      if (this->pref_.load(&index))
+        this->widget_->set_selected_index(index, LV_ANIM_OFF);
+    }
+    this->publish();
     lv_obj_add_event_cb(
         this->widget_->obj,
         [](lv_event_t *e) {
@@ -24,11 +32,6 @@ class LVGLSelect : public select::Select {
           it->set_options_();
         },
         LV_EVENT_REFRESH, this);
-    if (this->initial_state_.has_value()) {
-      this->control(this->initial_state_.value());
-      this->initial_state_.reset();
-    }
-    this->publish();
     auto lamb = [](lv_event_t *e) {
       auto *self = static_cast<LVGLSelect *>(e->user_data);
       self->publish();
@@ -37,21 +40,25 @@ class LVGLSelect : public select::Select {
     lv_obj_add_event_cb(this->widget_->obj, lamb, lv_update_event, this);
   }
 
-  void publish() { this->publish_state(this->widget_->get_selected_text()); }
+  void publish() {
+    this->publish_state(this->widget_->get_selected_text());
+    if (this->restore_) {
+      auto index = this->widget_->get_selected_index();
+      this->pref_.save(&index);
+    }
+  }
 
  protected:
   void control(const std::string &value) override {
-    if (this->widget_ != nullptr) {
-      this->widget_->set_selected_text(value, this->anim_);
-    } else {
-      this->initial_state_ = value;
-    }
+    this->widget_->set_selected_text(value, this->anim_);
+    this->publish();
   }
   void set_options_() { this->traits.set_options(this->widget_->get_options()); }
 
-  LvSelectable *widget_{};
-  optional<std::string> initial_state_{};
-  lv_anim_enable_t anim_{LV_ANIM_OFF};
+  LvSelectable *widget_;
+  lv_anim_enable_t anim_;
+  bool restore_;
+  ESPPreferenceObject pref_{};
 };
 
 }  // namespace lvgl
