@@ -38,6 +38,9 @@
 namespace esphome {
 namespace opentherm {
 
+static const uint8_t REPEATING_MESSAGE_ORDER = 255;
+static const uint8_t INITIAL_UNORDERED_MESSAGE_ORDER = 254;
+
 // OpenTherm component for ESPHome
 class OpenthermHub : public Component {
  protected:
@@ -58,15 +61,12 @@ class OpenthermHub : public Component {
 
   OPENTHERM_INPUT_SENSOR_LIST(OPENTHERM_DECLARE_INPUT_SENSOR, )
 
-  // The set of initial messages to send on starting communication with the boiler
-  std::vector<MessageId> initial_messages_;
-  // and the repeating messages which are sent repeatedly to update various sensors
-  // and boiler parameters (like the setpoint).
-  std::vector<MessageId> repeating_messages_;
-  // Indicates if we are still working on the initial requests or not
+  OPENTHERM_SETTING_LIST(OPENTHERM_DECLARE_SETTING, )
+
   bool sending_initial_ = true;
-  // Index for the current request in one of the _requests sets.
-  std::vector<MessageId>::const_iterator current_message_iterator_;
+  std::unordered_map<MessageId, uint8_t> configured_messages_;
+  std::vector<MessageId> messages_;
+  std::vector<MessageId>::const_iterator message_iterator_;
 
   uint32_t last_conversation_start_ = 0;
   uint32_t last_conversation_end_ = 0;
@@ -78,20 +78,25 @@ class OpenthermHub : public Component {
   // Very likely to happen while using Dallas temperature sensors.
   bool sync_mode_ = false;
 
-  float opentherm_version_ = 0.0f;
+  CallbackManager<void(OpenthermData &)> before_send_callback_;
+  CallbackManager<void(OpenthermData &)> before_process_response_callback_;
 
   // Create OpenTherm messages based on the message id
   OpenthermData build_request_(MessageId request_id) const;
-  void handle_protocol_write_error_();
-  void handle_protocol_read_error_();
+  bool handle_error_(OperationMode mode);
+  void handle_protocol_error_();
   void handle_timeout_error_();
+  void handle_timer_error_();
   void stop_opentherm_();
   void start_conversation_();
   void read_response_();
-  bool check_timings_(uint32_t cur_time);
+  void check_timings_(uint32_t cur_time);
   bool should_skip_loop_(uint32_t cur_time) const;
   void sync_loop_();
 
+  void write_initial_messages_(std::vector<MessageId> &target);
+  void write_repeating_messages_(std::vector<MessageId> &target);
+
   template<typename F> bool spin_wait_(uint32_t timeout, F func) {
     auto start_time = millis();
     while (func()) {
@@ -127,13 +132,18 @@ class OpenthermHub : public Component {
 
   OPENTHERM_INPUT_SENSOR_LIST(OPENTHERM_SET_INPUT_SENSOR, )
 
+  OPENTHERM_SETTING_LIST(OPENTHERM_SET_SETTING, )
+
   // Add a request to the vector of initial requests
-  void add_initial_message(MessageId message_id) { this->initial_messages_.push_back(message_id); }
+  void add_initial_message(MessageId message_id) {
+    this->configured_messages_[message_id] = INITIAL_UNORDERED_MESSAGE_ORDER;
+  }
+  void add_initial_message(MessageId message_id, uint8_t order) { this->configured_messages_[message_id] = order; }
   // Add a request to the set of repeating requests. Note that a large number of repeating
   // requests will slow down communication with the boiler. Each request may take up to 1 second,
   // so with all sensors enabled, it may take about half a minute before a change in setpoint
   // will be processed.
-  void add_repeating_message(MessageId message_id) { this->repeating_messages_.push_back(message_id); }
+  void add_repeating_message(MessageId message_id) { this->configured_messages_[message_id] = REPEATING_MESSAGE_ORDER; }
 
   // There are seven status variables, which can either be set as a simple variable,
   // or using a switch. ch_enable and dhw_enable default to true, the others to false.
@@ -149,7 +159,13 @@ class OpenthermHub : public Component {
   void set_summer_mode_active(bool value) { this->summer_mode_active = value; }
   void set_dhw_block(bool value) { this->dhw_block = value; }
   void set_sync_mode(bool sync_mode) { this->sync_mode_ = sync_mode; }
-  void set_opentherm_version(float value) { this->opentherm_version_ = value; }
+
+  void add_on_before_send_callback(std::function<void(OpenthermData &)> &&callback) {
+    this->before_send_callback_.add(std::move(callback));
+  }
+  void add_on_before_process_response_callback(std::function<void(OpenthermData &)> &&callback) {
+    this->before_process_response_callback_.add(std::move(callback));
+  }
 
   float get_setup_priority() const override { return setup_priority::HARDWARE; }
 
