@@ -10,11 +10,11 @@ static const char *const TAG = "pipsolar.switch";
 void PipsolarSwitch::dump_config() { LOG_SWITCH("", "Pipsolar Switch", this); }
 void PipsolarSwitch::write_state(bool state) {
   if (state) {
-    if (this->on_command_.length() > 0) {
+    if (!this->on_command_.empty()) {
       this->parent_->switch_command(this->on_command_);
     }
   } else {
-    if (this->off_command_.length() > 0) {
+    if (!this->off_command_.empty()) {
       this->parent_->switch_command(this->off_command_);
     }
   }
