@@ -21,7 +21,7 @@ void ShutdownSwitch::write_state(bool state) {
   this->publish_state(false);
 
   if (state) {
-    ESP_LOGI(TAG, "Shutting down...");
+    ESP_LOGI(TAG, "Shutting down");
     delay(100);  // NOLINT
 
     App.run_safe_shutdown_hooks();
