@@ -1,8 +1,7 @@
 import esphome.codegen as cg
+from esphome.components import light, spi
 import esphome.config_validation as cv
-from esphome.components import light
-from esphome.components import spi
-from esphome.const import CONF_OUTPUT_ID, CONF_NUM_LEDS
+from esphome.const import CONF_NUM_LEDS, CONF_OUTPUT_ID
 
 spi_led_strip_ns = cg.esphome_ns.namespace("spi_led_strip")
 SpiLedStrip = spi_led_strip_ns.class_(
@@ -18,8 +17,7 @@ CONFIG_SCHEMA = light.ADDRESSABLE_LIGHT_SCHEMA.extend(
 
 
 async def to_code(config):
-    var = cg.new_Pvariable(config[CONF_OUTPUT_ID])
-    cg.add(var.set_num_leds(config[CONF_NUM_LEDS]))
+    var = cg.new_Pvariable(config[CONF_OUTPUT_ID], config[CONF_NUM_LEDS])
     await light.register_light(var, config)
     await spi.register_spi_device(var, config)
     await cg.register_component(var, config)
