@@ -9,10 +9,14 @@
 #include "esphome/core/hal.h"
 #include "esphome/core/helpers.h"
 #include "esphome/core/log.h"
+#ifdef USE_RUNTIME_STATS
+#include "esphome/components/runtime_stats/runtime_stats.h"
+#endif
 
 namespace esphome {
 
 static const char *const TAG = "component";
+static const char *const UNSPECIFIED_MESSAGE = "unspecified";
 
 // Global vectors for component data that doesn't belong in every instance.
 // Using vector instead of unordered_map for both because:
@@ -129,7 +133,7 @@ void Component::call_dump_config() {
   this->dump_config();
   if (this->is_failed()) {
     // Look up error message from global vector
-    const char *error_msg = "unspecified";
+    const char *error_msg = nullptr;
     if (component_error_messages) {
       for (const auto &pair : *component_error_messages) {
         if (pair.first == this) {
@@ -138,7 +142,8 @@ void Component::call_dump_config() {
         }
       }
     }
-    ESP_LOGE(TAG, "  %s is marked FAILED: %s", this->get_component_source(), error_msg);
+    ESP_LOGE(TAG, "  %s is marked FAILED: %s", this->get_component_source(),
+             error_msg ? error_msg : UNSPECIFIED_MESSAGE);
   }
 }
 
@@ -146,28 +151,33 @@ uint8_t Component::get_component_state() const { return this->component_state_;
 void Component::call() {
   uint8_t state = this->component_state_ & COMPONENT_STATE_MASK;
   switch (state) {
-    case COMPONENT_STATE_CONSTRUCTION:
+    case COMPONENT_STATE_CONSTRUCTION: {
       // State Construction: Call setup and set state to setup
-      this->component_state_ &= ~COMPONENT_STATE_MASK;
-      this->component_state_ |= COMPONENT_STATE_SETUP;
+      this->set_component_state_(COMPONENT_STATE_SETUP);
+      ESP_LOGV(TAG, "Setup %s", this->get_component_source());
+#if ESPHOME_LOG_LEVEL >= ESPHOME_LOG_LEVEL_DEBUG
+      uint32_t start_time = millis();
+#endif
       this->call_setup();
+#if ESPHOME_LOG_LEVEL >= ESPHOME_LOG_LEVEL_DEBUG
+      uint32_t setup_time = millis() - start_time;
+      ESP_LOGCONFIG(TAG, "Setup %s took %ums", this->get_component_source(), (unsigned) setup_time);
+#endif
       break;
+    }
     case COMPONENT_STATE_SETUP:
       // State setup: Call first loop and set state to loop
-      this->component_state_ &= ~COMPONENT_STATE_MASK;
-      this->component_state_ |= COMPONENT_STATE_LOOP;
+      this->set_component_state_(COMPONENT_STATE_LOOP);
       this->call_loop();
       break;
     case COMPONENT_STATE_LOOP:
       // State loop: Call loop
       this->call_loop();
       break;
-    case COMPONENT_STATE_FAILED:  // NOLINT(bugprone-branch-clone)
+    case COMPONENT_STATE_FAILED:
       // State failed: Do nothing
-      break;
-    case COMPONENT_STATE_LOOP_DONE:  // NOLINT(bugprone-branch-clone)
+    case COMPONENT_STATE_LOOP_DONE:
       // State loop done: Do nothing, component has finished its work
-      break;
     default:
       break;
   }
@@ -192,25 +202,26 @@ bool Component::should_warn_of_blocking(uint32_t blocking_time) {
 }
 void Component::mark_failed() {
   ESP_LOGE(TAG, "%s was marked as failed", this->get_component_source());
-  this->component_state_ &= ~COMPONENT_STATE_MASK;
-  this->component_state_ |= COMPONENT_STATE_FAILED;
+  this->set_component_state_(COMPONENT_STATE_FAILED);
   this->status_set_error();
   // Also remove from loop since failed components shouldn't loop
   App.disable_component_loop_(this);
 }
+void Component::set_component_state_(uint8_t state) {
+  this->component_state_ &= ~COMPONENT_STATE_MASK;
+  this->component_state_ |= state;
+}
 void Component::disable_loop() {
   if ((this->component_state_ & COMPONENT_STATE_MASK) != COMPONENT_STATE_LOOP_DONE) {
     ESP_LOGVV(TAG, "%s loop disabled", this->get_component_source());
-    this->component_state_ &= ~COMPONENT_STATE_MASK;
-    this->component_state_ |= COMPONENT_STATE_LOOP_DONE;
+    this->set_component_state_(COMPONENT_STATE_LOOP_DONE);
     App.disable_component_loop_(this);
   }
 }
 void Component::enable_loop() {
   if ((this->component_state_ & COMPONENT_STATE_MASK) == COMPONENT_STATE_LOOP_DONE) {
     ESP_LOGVV(TAG, "%s loop enabled", this->get_component_source());
-    this->component_state_ &= ~COMPONENT_STATE_MASK;
-    this->component_state_ |= COMPONENT_STATE_LOOP;
+    this->set_component_state_(COMPONENT_STATE_LOOP);
     App.enable_component_loop_(this);
   }
 }
@@ -230,8 +241,7 @@ void IRAM_ATTR HOT Component::enable_loop_soon_any_context() {
 void Component::reset_to_construction_state() {
   if ((this->component_state_ & COMPONENT_STATE_MASK) == COMPONENT_STATE_FAILED) {
     ESP_LOGI(TAG, "%s is being reset to construction state", this->get_component_source());
-    this->component_state_ &= ~COMPONENT_STATE_MASK;
-    this->component_state_ |= COMPONENT_STATE_CONSTRUCTION;
+    this->set_component_state_(COMPONENT_STATE_CONSTRUCTION);
     // Clear error status when resetting
     this->status_clear_error();
   }
@@ -276,15 +286,15 @@ void Component::status_set_warning(const char *message) {
     return;
   this->component_state_ |= STATUS_LED_WARNING;
   App.app_state_ |= STATUS_LED_WARNING;
-  ESP_LOGW(TAG, "%s set Warning flag: %s", this->get_component_source(), message);
+  ESP_LOGW(TAG, "%s set Warning flag: %s", this->get_component_source(), message ? message : UNSPECIFIED_MESSAGE);
 }
 void Component::status_set_error(const char *message) {
   if ((this->component_state_ & STATUS_LED_ERROR) != 0)
     return;
   this->component_state_ |= STATUS_LED_ERROR;
   App.app_state_ |= STATUS_LED_ERROR;
-  ESP_LOGE(TAG, "%s set Error flag: %s", this->get_component_source(), message);
-  if (strcmp(message, "unspecified") != 0) {
+  ESP_LOGE(TAG, "%s set Error flag: %s", this->get_component_source(), message ? message : UNSPECIFIED_MESSAGE);
+  if (message != nullptr) {
     // Lazy allocate the error messages vector if needed
     if (!component_error_messages) {
       component_error_messages = std::make_unique<std::vector<std::pair<const Component *, const char *>>>();
@@ -370,11 +380,10 @@ bool Component::has_overridden_loop() const {
 PollingComponent::PollingComponent(uint32_t update_interval) : update_interval_(update_interval) {}
 
 void PollingComponent::call_setup() {
+  // init the poller before calling setup, allowing setup to cancel it if desired
+  this->start_poller();
   // Let the polling component subclass setup their HW.
   this->setup();
-
-  // init the poller
-  this->start_poller();
 }
 
 void PollingComponent::start_poller() {
@@ -396,6 +405,13 @@ uint32_t WarnIfComponentBlockingGuard::finish() {
   uint32_t curr_time = millis();
 
   uint32_t blocking_time = curr_time - this->started_;
+
+#ifdef USE_RUNTIME_STATS
+  // Record component runtime stats
+  if (global_runtime_stats != nullptr) {
+    global_runtime_stats->record_component_time(this->component_, blocking_time, curr_time);
+  }
+#endif
   bool should_warn;
   if (this->component_ != nullptr) {
     should_warn = this->component_->should_warn_of_blocking(blocking_time);
